@@ -129,7 +129,7 @@ TEST(StanUiCommand, refresh_zero_ok) {
                         " sample num_samples=10 num_warmup=10 init=0 output "
                         "refresh=0 file=test/output.csv";
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::OK), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::OK), out.err_code);
   EXPECT_EQ(0, count_matches("Iteration:", out.output));
 }
 
@@ -144,7 +144,7 @@ TEST(StanUiCommand, refresh_nonzero_ok) {
                         " sample num_samples=10 num_warmup=10 init=0 output "
                         "refresh=1 file=test/output.csv";
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::OK), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::OK), out.err_code);
   EXPECT_EQ(20, count_matches("Iteration:", out.output));
 }
 
@@ -160,7 +160,7 @@ TEST(StanUiCommand, zero_init_value_fail) {
   std::string command = convert_model_path(model_path)
                         + " sample init=0 output file=test/output.csv";
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::SOFTWARE), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::NOT_OK), out.err_code);
 
   EXPECT_TRUE(out.header.length() > 0U);
   EXPECT_TRUE(out.body.length() > 0U);
@@ -182,7 +182,7 @@ TEST(StanUiCommand, zero_init_domain_fail) {
                         + " sample init=0 output file=test/output.csv";
 
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::SOFTWARE), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::NOT_OK), out.err_code);
 
   EXPECT_TRUE(out.header.length() > 0U);
   EXPECT_TRUE(out.body.length() > 0U);
@@ -211,7 +211,7 @@ TEST(StanUiCommand, user_init_value_fail) {
                         + " output file=test/output.csv";
 
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::SOFTWARE), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::NOT_OK), out.err_code);
 
   EXPECT_TRUE(out.header.length() > 0U);
   EXPECT_TRUE(out.body.length() > 0U);
@@ -240,7 +240,7 @@ TEST(StanUiCommand, user_init_domain_fail) {
                         + " output file=test/output.csv";
 
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::SOFTWARE), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::NOT_OK), out.err_code);
 
   EXPECT_TRUE(out.header.length() > 0U);
   EXPECT_TRUE(out.body.length() > 0U);
@@ -249,7 +249,7 @@ TEST(StanUiCommand, user_init_domain_fail) {
       << "Failed running: " << out.command;
 }
 
-TEST(StanUiCommand, CheckCommand_default) {
+TEST(StanUiCommand, CheckCommand_no_args) {
   std::vector<std::string> model_path;
   model_path.push_back("src");
   model_path.push_back("test");
@@ -258,7 +258,7 @@ TEST(StanUiCommand, CheckCommand_default) {
 
   std::string command = convert_model_path(model_path);
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::USAGE), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::NOT_OK), out.err_code);
 }
 
 TEST(StanUiCommand, CheckCommand_help) {
@@ -271,7 +271,7 @@ TEST(StanUiCommand, CheckCommand_help) {
   std::string command = convert_model_path(model_path) + " help";
 
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::OK), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::OK), out.err_code);
 }
 
 TEST(StanUiCommand, CheckCommand_unrecognized_argument) {
@@ -284,7 +284,7 @@ TEST(StanUiCommand, CheckCommand_unrecognized_argument) {
   std::string command = convert_model_path(model_path) + " foo";
 
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::USAGE), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::NOT_OK), out.err_code);
 }
 
 TEST(StanUiCommand, timing_info) {
@@ -298,7 +298,7 @@ TEST(StanUiCommand, timing_info) {
                         " sample num_samples=10 num_warmup=10 init=0 output "
                         "refresh=0 file=test/output.csv";
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::OK), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::OK), out.err_code);
 
   std::fstream output_csv_stream("test/output.csv");
   std::stringstream output_sstream;
@@ -323,7 +323,7 @@ TEST(StanUiCommand, run_info) {
                         " sample num_samples=10 num_warmup=10 init=0 output "
                         "refresh=0 file=test/output.csv";
   run_command_output out = run_command(command);
-  EXPECT_EQ(int(stan::services::error_codes::OK), out.err_code);
+  EXPECT_EQ(int(cmdstan::return_codes::OK), out.err_code);
 
   std::fstream output_csv_stream("test/output.csv");
   std::stringstream output_sstream;
