@@ -351,7 +351,7 @@ def benchmark(
             if "matrix" in arg:
                 arg_dimm = 2
             if "[" in arg:
-                arg_dimm += len(arg.split("[")[1])
+                arg_dimm += len(arg.split("]")[0].split("[")[1])
             if arg_dimm == dimm:
                 args_with_max_dimm += 1
             elif arg_dimm > dimm:
@@ -403,10 +403,10 @@ def benchmark(
                     n,
                     (arg_overload, cpp_arg_template, stan_arg),
             ) in enumerate(zip(arg_overloads, cpp_arg_templates, stan_args)):
-                if stan_arg.endswith("]"):
-                    stan_arg2, vec = stan_arg.split("[")
+                n_vec, inner_type = parse_array(stan_arg)
+                if n_vec:
                     benchmark_name += (
-                            "_" + arg_overload + "_" + stan_arg2 + str(len(vec))
+                            "_" + arg_overload + "_" + inner_type + str(n_vec)
                     )
                 else:
                     benchmark_name += "_" + arg_overload + "_" + stan_arg
@@ -480,11 +480,12 @@ def benchmark(
                     code += "stan::math::to_var_value({}), ".format(var_name)
                 else:
                     code += var_name + ", "
-            if opencl == "base":
-                var_conversions += "  stan::math::opencl_context.queue().finish();\n"
             code = code[:-2] + "));\n"
             if "Rev" in arg_overloads:
                 code += "    stan::math::grad();\n"
+            if opencl == "base":
+              code += "    stan::math::opencl_context.queue().finish();\n"
+              var_conversions += "    stan::math::opencl_context.queue().finish();\n"
             result += BENCHMARK_TEMPLATE.format(
                 benchmark_name=benchmark_name,
                 setup=setup,
