@@ -4,7 +4,6 @@
 #include <stan/math/prim.hpp>
 #include <stan/math/rev.hpp>
 #include <stan/model/indexing/index.hpp>
-#include <stan/model/indexing/index_list.hpp>
 #include <stan/model/indexing/rvalue_at.hpp>
 #include <stan/model/indexing/rvalue_index_size.hpp>
 #include <stan/model/indexing/access_helpers.hpp>
@@ -52,11 +51,20 @@ namespace model {
  * @return Input value.
  */
 template <typename T>
-inline T rvalue(T&& x, const nil_index_list& /*idx*/, const char* /*name*/ = "",
-                int /*depth*/ = 0) {
+inline T rvalue(T&& x, const char* /*name*/) {
   return std::forward<T>(x);
 }
 
+template <typename T>
+inline T& rvalue(T& x, const char* /*name*/) {
+  return x;
+}
+
+template <typename T>
+inline const T& rvalue(const T& x, const char* /*name*/) {
+  return x;
+}
+
 /**
  * Return the result of indexing a type without taking a subset. Mostly used as
  * an intermediary rvalue function when doing multiple subsets.
@@ -65,17 +73,23 @@ inline T rvalue(T&& x, const nil_index_list& /*idx*/, const char* /*name*/ = "",
  *
  * @tparam T A type that is a plain object.
  * @param[in] x an object.
- * @param[in] idxs Index consisting of one omni-index.
- * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
  * @return Result of indexing matrix.
  */
 template <typename T>
-inline T rvalue(T&& x, const cons_index_list<index_omni, nil_index_list>& idxs,
-                const char* name = "ANON", int depth = 0) {
+inline T rvalue(T&& x, const char* /*name*/, index_omni /*idx*/) {
   return std::forward<T>(x);
 }
 
+template <typename T>
+inline T& rvalue(T& x, const char* /*name*/, index_omni /*idx*/) {
+  return x;
+}
+
+template <typename T>
+inline const T& rvalue(const T& x, const char* /*name*/, index_omni /*idx*/) {
+  return x;
+}
+
 /**
  * Return the result of indexing a type without taking a subset
  *
@@ -83,20 +97,27 @@ inline T rvalue(T&& x, const cons_index_list<index_omni, nil_index_list>& idxs,
  *
  * @tparam T Any type.
  * @param[in] x an object.
- * @param[in] idxs Index consisting of two omni-indices.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
  * @return Result of indexing matrix.
  */
 template <typename T>
-inline T rvalue(
-    T&& x,
-    const cons_index_list<index_omni,
-                          cons_index_list<index_omni, nil_index_list>>& idxs,
-    const char* name = "ANON", int depth = 0) {
+inline T rvalue(T&& x, const char* name, index_omni /*idx1*/,
+                index_omni /*idx2*/) {
   return std::forward<T>(x);
 }
 
+template <typename T>
+inline T& rvalue(T& x, const char* name, index_omni /*idx1*/,
+                 index_omni /*idx2*/) {
+  return x;
+}
+
+template <typename T>
+inline const T& rvalue(const T& x, const char* name, index_omni /*idx1*/,
+                       index_omni /*idx2*/) {
+  return x;
+}
+
 /**
  * Return a single element of a Vector.
  *
@@ -104,19 +125,16 @@ inline T rvalue(
  *
  * @tparam Vec An Eigen vector or `var_value<T>` where `T` is an eigen vector.
  * @param[in] v Vector being indexed.
- * @param[in] idxs One single index.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] idx One single index.
  * @return Result of indexing vector.
  */
 template <typename Vec, require_vector_t<Vec>* = nullptr,
           require_not_std_vector_t<Vec>* = nullptr>
-inline auto rvalue(Vec&& v,
-                   const cons_index_list<index_uni, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
+inline auto rvalue(Vec&& v, const char* name, index_uni idx) {
   using stan::math::to_ref;
-  math::check_range("vector[uni] indexing", name, v.size(), idxs.head_.n_);
-  return v.coeff(idxs.head_.n_ - 1);
+  math::check_range("vector[uni] indexing", name, v.size(), idx.n_);
+  return v.coeff(idx.n_ - 1);
 }
 
 /**
@@ -126,24 +144,21 @@ inline auto rvalue(Vec&& v,
  *
  * @tparam EigVec Eigen type with either dynamic rows or columns, but not both.
  * @param[in] v Eigen vector type.
- * @param[in] idxs Sequence of integers.
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
+ * @param[in] name Name of variable
+ * @param[in] idx Sequence of integers.
  * @throw std::out_of_range If any of the indices are out of bounds.
  * @throw std::invalid_argument If the value size isn't the same as
  * the indexed size.
  */
 template <typename EigVec, require_eigen_vector_t<EigVec>* = nullptr>
-inline plain_type_t<EigVec> rvalue(
-    EigVec&& v, const cons_index_list<index_multi, nil_index_list>& idxs,
-    const char* name = "ANON", int depth = 0) {
+inline plain_type_t<EigVec> rvalue(EigVec&& v, const char* name,
+                                   const index_multi& idx) {
   const auto v_size = v.size();
   const auto& v_ref = stan::math::to_ref(v);
-  plain_type_t<EigVec> ret_v(idxs.head_.ns_.size());
-  for (int i = 0; i < idxs.head_.ns_.size(); ++i) {
-    math::check_range("vector[multi] indexing", name, v_ref.size(),
-                      idxs.head_.ns_[i]);
-    ret_v.coeffRef(i) = v_ref.coeff(idxs.head_.ns_[i] - 1);
+  plain_type_t<EigVec> ret_v(idx.ns_.size());
+  for (int i = 0; i < idx.ns_.size(); ++i) {
+    math::check_range("vector[multi] indexing", name, v_ref.size(), idx.ns_[i]);
+    ret_v.coeffRef(i) = v_ref.coeff(idx.ns_[i] - 1);
   }
   return ret_v;
 }
@@ -155,27 +170,22 @@ inline plain_type_t<EigVec> rvalue(
  *
  * @tparam Vec An Eigen vector or `var_value<T>` where `T` is an eigen vector.
  * @param[in] v Vector being indexed.
- * @param[in] idxs An index to select from a minimum to maximum range.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] idx An index to select from a minimum to maximum range.
  * @return Result of indexing vector.
  */
 template <typename Vec, require_vector_t<Vec>* = nullptr,
           require_not_std_vector_t<Vec>* = nullptr>
-inline auto rvalue(Vec&& v,
-                   const cons_index_list<index_min_max, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
-  math::check_range("vector[min_max] min indexing", name, v.size(),
-                    idxs.head_.min_);
-  math::check_range("vector[min_max] max indexing", name, v.size(),
-                    idxs.head_.max_);
-  if (idxs.head_.is_ascending()) {
-    const auto slice_start = idxs.head_.min_ - 1;
-    const auto slice_size = idxs.head_.max_ - slice_start;
+inline auto rvalue(Vec&& v, const char* name, index_min_max idx) {
+  math::check_range("vector[min_max] min indexing", name, v.size(), idx.min_);
+  math::check_range("vector[min_max] max indexing", name, v.size(), idx.max_);
+  if (idx.is_ascending()) {
+    const auto slice_start = idx.min_ - 1;
+    const auto slice_size = idx.max_ - slice_start;
     return v.segment(slice_start, slice_size).eval();
   } else {
-    const auto slice_start = idxs.head_.max_ - 1;
-    const auto slice_size = idxs.head_.min_ - slice_start;
+    const auto slice_start = idx.max_ - 1;
+    const auto slice_size = idx.min_ - slice_start;
     return v.segment(slice_start, slice_size).reverse().eval();
   }
 }
@@ -187,20 +197,16 @@ inline auto rvalue(Vec&& v,
  *
  * @tparam Vec An Eigen vector or `var_value<T>` where `T` is an eigen vector.
  * @param[in] x vector
- * @param[in] idxs An indexing from a specific minimum index to the end out
+ * @param[in] name Name of variable
+ * @param[in] idx An indexing from a specific minimum index to the end out
  *  of a bottom row of a matrix
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename Vec, require_vector_t<Vec>* = nullptr,
           require_not_std_vector_t<Vec>* = nullptr>
-inline auto rvalue(Vec&& x,
-                   const cons_index_list<index_min, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
-  stan::math::check_range("vector[min] indexing", name, x.size(),
-                          idxs.head_.min_);
-  return x.tail(x.size() - idxs.head_.min_ + 1);
+inline auto rvalue(Vec&& x, const char* name, index_min idx) {
+  stan::math::check_range("vector[min] indexing", name, x.size(), idx.min_);
+  return x.tail(x.size() - idx.min_ + 1);
 }
 
 /**
@@ -210,20 +216,16 @@ inline auto rvalue(Vec&& x,
  *
  * @tparam Vec An Eigen vector or `var_value<T>` where `T` is an eigen vector.
  * @param[in] x vector.
- * @param[in] idxs An indexing from the start of the container up to
+ * @param[in] name Name of variable
+ * @param[in] idx An indexing from the start of the container up to
  * the specified maximum index (inclusive).
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename Vec, require_vector_t<Vec>* = nullptr,
           require_not_std_vector_t<Vec>* = nullptr>
-inline auto rvalue(Vec&& x,
-                   const cons_index_list<index_max, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
-  stan::math::check_range("vector[max] indexing", name, x.size(),
-                          idxs.head_.max_);
-  return x.head(idxs.head_.max_);
+inline auto rvalue(Vec&& x, const char* name, index_max idx) {
+  stan::math::check_range("vector[max] indexing", name, x.size(), idx.max_);
+  return x.head(idx.max_);
 }
 
 /**
@@ -235,17 +237,14 @@ inline auto rvalue(Vec&& x,
  * @tparam Mat An eigen matrix or `var_value<T>` whose inner type is an Eigen
  * matrix.
  * @param[in] x matrix.
- * @param[in] idxs Index consisting of one uni-index.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] idx uni-index
  * @return Result of indexing matrix.
  */
 template <typename Mat, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(Mat&& x,
-                   const cons_index_list<index_uni, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
-  math::check_range("matrix[uni] indexing", name, x.rows(), idxs.head_.n_);
-  return x.row(idxs.head_.n_ - 1);
+inline auto rvalue(Mat&& x, const char* name, index_uni idx) {
+  math::check_range("matrix[uni] indexing", name, x.rows(), idx.n_);
+  return x.row(idx.n_ - 1);
 }
 
 /**
@@ -255,19 +254,17 @@ inline auto rvalue(Mat&& x,
  *
  * @tparam EigMat Eigen type with dynamic rows and columns.
  * @param[in] x Eigen type
- * @param[in] idxs A multi index for selecting a set of rows.
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
+ * @param[in] name Name of variable
+ * @param[in] idx A multi index for selecting a set of rows.
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename EigMat, require_eigen_dense_dynamic_t<EigMat>* = nullptr>
-inline plain_type_t<EigMat> rvalue(
-    EigMat&& x, const cons_index_list<index_multi, nil_index_list>& idxs,
-    const char* name = "ANON", int depth = 0) {
+inline plain_type_t<EigMat> rvalue(EigMat&& x, const char* name,
+                                   const index_multi& idx) {
   const auto& x_ref = stan::math::to_ref(x);
-  plain_type_t<EigMat> x_ret(idxs.head_.ns_.size(), x.cols());
-  for (int i = 0; i < idxs.head_.ns_.size(); ++i) {
-    const int n = idxs.head_.ns_[i];
+  plain_type_t<EigMat> x_ret(idx.ns_.size(), x.cols());
+  for (int i = 0; i < idx.ns_.size(); ++i) {
+    const int n = idx.ns_[i];
     math::check_range("matrix[multi] row indexing", name, x_ref.rows(), n);
     x_ret.row(i) = x_ref.row(n - 1);
   }
@@ -283,18 +280,14 @@ inline plain_type_t<EigMat> rvalue(
  * @tparam Mat An eigen matrix or `var_value<T>` whose inner type is an Eigen
  * matrix.
  * @param[in] x matrix.
- * @param[in] idxs Index consisting of one uni-index.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] idx index_min
  * @return Result of indexing matrix.
  */
 template <typename Mat, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(Mat&& x,
-                   const cons_index_list<index_min, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
-  const auto row_size = x.rows() - (idxs.head_.min_ - 1);
-  math::check_range("matrix[min] row indexing", name, x.rows(),
-                    idxs.head_.min_);
+inline auto rvalue(Mat&& x, const char* name, index_min idx) {
+  const auto row_size = x.rows() - (idx.min_ - 1);
+  math::check_range("matrix[min] row indexing", name, x.rows(), idx.min_);
   return x.bottomRows(row_size);
 }
 
@@ -306,19 +299,15 @@ inline auto rvalue(Mat&& x,
  * @tparam Mat An eigen matrix or `var_value<T>` whose inner type is an Eigen
  * matrix.
  * @param[in] x matrix
- * @param[in] idxs An indexing from the start of the container up to
+ * @param[in] name Name of variable
+ * @param[in] idx An indexing from the start of the container up to
  * the specified maximum index (inclusive).
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename Mat, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(Mat&& x,
-                   const cons_index_list<index_max, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
-  math::check_range("matrix[max] row indexing", name, x.rows(),
-                    idxs.head_.max_);
-  return x.topRows(idxs.head_.max_);
+inline auto rvalue(Mat&& x, const char* name, index_max idx) {
+  math::check_range("matrix[max] row indexing", name, x.rows(), idx.max_);
+  return x.topRows(idx.max_);
 }
 
 /**
@@ -329,26 +318,22 @@ inline auto rvalue(Mat&& x,
  * @tparam Mat An eigen matrix or `var_value<T>` whose inner type is an Eigen
  * matrix.
  * @param[in] x Dense matrix
- * @param[in] idxs A min max index to select a range of rows.
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
+ * @param[in] name Name of variable
+ * @param[in] idx A min max index to select a range of rows.
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename Mat, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(Mat&& x,
-                   const cons_index_list<index_min_max, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
+inline auto rvalue(Mat&& x, const char* name, index_min_max idx) {
   math::check_range("matrix[min_max] max row indexing", name, x.rows(),
-                    idxs.head_.max_);
+                    idx.max_);
   math::check_range("matrix[min_max] min row indexing", name, x.rows(),
-                    idxs.head_.min_);
-  if (idxs.head_.is_ascending()) {
-    const auto row_size = idxs.head_.max_ - idxs.head_.min_ + 1;
-    return x.middleRows(idxs.head_.min_ - 1, row_size).eval();
+                    idx.min_);
+  if (idx.is_ascending()) {
+    const auto row_size = idx.max_ - idx.min_ + 1;
+    return x.middleRows(idx.min_ - 1, row_size).eval();
   } else {
-    const auto row_size = idxs.head_.min_ - idxs.head_.max_ + 1;
-    return internal::colwise_reverse(
-               x.middleRows(idxs.head_.max_ - 1, row_size))
+    const auto row_size = idx.min_ - idx.max_ + 1;
+    return internal::colwise_reverse(x.middleRows(idx.max_ - 1, row_size))
         .eval();
   }
 }
@@ -362,51 +347,48 @@ inline auto rvalue(Mat&& x,
  * @tparam Mat An eigen matrix or `var_value<T>` whose inner type is an Eigen
  * matrix.
  * @param[in] x Eigen matrix.
- * @param[in] idxs Two min max indices for selecting a block of the matrix.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] row_idx Min max index for selecting rows.
+ * @param[in] col_idx Min max index for selecting cols.
  * @return Result of indexing matrix.
  */
 template <typename Mat, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(
-    Mat&& x,
-    const cons_index_list<index_min_max,
-                          cons_index_list<index_min_max, nil_index_list>>& idxs,
-    const char* name = "ANON", int depth = 0) {
+inline auto rvalue(Mat&& x, const char* name, index_min_max row_idx,
+                   index_min_max col_idx) {
   math::check_range("matrix[min_max, min_max] min row indexing", name, x.rows(),
-                    idxs.head_.min_);
+                    row_idx.min_);
   math::check_range("matrix[min_max, min_max] max row indexing", name, x.rows(),
-                    idxs.head_.max_);
+                    row_idx.max_);
   math::check_range("matrix[min_max, min_max] min column indexing", name,
-                    x.cols(), idxs.tail_.head_.min_);
+                    x.cols(), col_idx.min_);
   math::check_range("matrix[min_max, min_max] max column indexing", name,
-                    x.cols(), idxs.tail_.head_.max_);
-  if (idxs.head_.is_ascending()) {
-    if (idxs.tail_.head_.is_ascending()) {
+                    x.cols(), col_idx.max_);
+  if (row_idx.is_ascending()) {
+    if (col_idx.is_ascending()) {
       return x
-          .block(idxs.head_.min_ - 1, idxs.tail_.head_.min_ - 1,
-                 idxs.head_.max_ - (idxs.head_.min_ - 1),
-                 idxs.tail_.head_.max_ - (idxs.tail_.head_.min_ - 1))
+          .block(row_idx.min_ - 1, col_idx.min_ - 1,
+                 row_idx.max_ - (row_idx.min_ - 1),
+                 col_idx.max_ - (col_idx.min_ - 1))
           .eval();
     } else {
       return internal::rowwise_reverse(
-                 x.block(idxs.head_.min_ - 1, idxs.tail_.head_.max_ - 1,
-                         idxs.head_.max_ - (idxs.head_.min_ - 1),
-                         idxs.tail_.head_.min_ - (idxs.tail_.head_.max_ - 1)))
+                 x.block(row_idx.min_ - 1, col_idx.max_ - 1,
+                         row_idx.max_ - (row_idx.min_ - 1),
+                         col_idx.min_ - (col_idx.max_ - 1)))
           .eval();
     }
   } else {
-    if (idxs.tail_.head_.is_ascending()) {
+    if (col_idx.is_ascending()) {
       return internal::colwise_reverse(
-                 x.block(idxs.head_.max_ - 1, idxs.tail_.head_.min_ - 1,
-                         idxs.head_.min_ - (idxs.head_.max_ - 1),
-                         idxs.tail_.head_.max_ - (idxs.tail_.head_.min_ - 1)))
+                 x.block(row_idx.max_ - 1, col_idx.min_ - 1,
+                         row_idx.min_ - (row_idx.max_ - 1),
+                         col_idx.max_ - (col_idx.min_ - 1)))
           .eval();
     } else {
       return x
-          .block(idxs.head_.max_ - 1, idxs.tail_.head_.max_ - 1,
-                 idxs.head_.min_ - (idxs.head_.max_ - 1),
-                 idxs.tail_.head_.min_ - (idxs.tail_.head_.max_ - 1))
+          .block(row_idx.max_ - 1, col_idx.max_ - 1,
+                 row_idx.min_ - (row_idx.max_ - 1),
+                 col_idx.min_ - (col_idx.max_ - 1))
           .reverse()
           .eval();
     }
@@ -421,22 +403,18 @@ inline auto rvalue(
  * @tparam Mat An eigen matrix or `var_value<T>` whose inner type is an Eigen
  * matrix.
  * @param[in] x Matrix to index.
- * @param[in] idxs Pair of single indexes.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] row_idx uni index for selecting rows.
+ * @param[in] col_idx uni index for selecting cols.
  * @return Result of indexing matrix.
  */
 template <typename Mat, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(
-    Mat&& x,
-    const cons_index_list<index_uni,
-                          cons_index_list<index_uni, nil_index_list>>& idxs,
-    const char* name = "ANON", int depth = 0) {
-  math::check_range("matrix[uni,uni] row indexing", name, x.rows(),
-                    idxs.head_.n_);
+inline auto rvalue(Mat&& x, const char* name, index_uni row_idx,
+                   index_uni col_idx) {
+  math::check_range("matrix[uni,uni] row indexing", name, x.rows(), row_idx.n_);
   math::check_range("matrix[uni,uni] column indexing", name, x.cols(),
-                    idxs.tail_.head_.n_);
-  return x.coeff(idxs.head_.n_ - 1, idxs.tail_.head_.n_ - 1);
+                    col_idx.n_);
+  return x.coeff(row_idx.n_ - 1, col_idx.n_ - 1);
 }
 
 /**
@@ -446,27 +424,24 @@ inline auto rvalue(
  *
  * @tparam EigMat Eigen type with dynamic rows and columns.
  * @param[in] x Matrix to index.
- * @param[in] idxs A uni index for the row and multi index for the columns
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
+ * @param[in] name Name of variable
+ * @param[in] row_idx uni index for selecting rows.
+ * @param[in] col_idx multi index for selecting cols.
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename EigMat, require_eigen_dense_dynamic_t<EigMat>* = nullptr>
 inline Eigen::Matrix<value_type_t<EigMat>, 1, Eigen::Dynamic> rvalue(
-    EigMat&& x,
-    const cons_index_list<index_uni,
-                          cons_index_list<index_multi, nil_index_list>>& idxs,
-    const char* name = "ANON", int depth = 0) {
+    EigMat&& x, const char* name, index_uni row_idx,
+    const index_multi& col_idx) {
   math::check_range("matrix[uni, multi] row indexing", name, x.rows(),
-                    idxs.head_.n_);
+                    row_idx.n_);
   const auto& x_ref = stan::math::to_ref(x);
   Eigen::Matrix<value_type_t<EigMat>, 1, Eigen::Dynamic> x_ret(
-      1, idxs.tail_.head_.ns_.size());
-  for (int i = 0; i < idxs.tail_.head_.ns_.size(); ++i) {
+      1, col_idx.ns_.size());
+  for (int i = 0; i < col_idx.ns_.size(); ++i) {
     math::check_range("matrix[uni, multi] column indexing", name, x.cols(),
-                      idxs.tail_.head_.ns_[i]);
-    x_ret.coeffRef(i)
-        = x_ref.coeff(idxs.head_.n_ - 1, idxs.tail_.head_.ns_[i] - 1);
+                      col_idx.ns_[i]);
+    x_ret.coeffRef(i) = x_ref.coeff(row_idx.n_ - 1, col_idx.ns_[i] - 1);
   }
   return x_ret;
 }
@@ -479,27 +454,24 @@ inline Eigen::Matrix<value_type_t<EigMat>, 1, Eigen::Dynamic> rvalue(
  *
  * @tparam EigMat Eigen type with dynamic rows and columns.
  * @param[in] x Matrix to index.
- * @param[in] idxs A multi index for the rows and a uni index for the column
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
+ * @param[in] name Name of variable
+ * @param[in] row_idx multi index for selecting rows.
+ * @param[in] col_idx uni index for selecting cols.
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename EigMat, require_eigen_dense_dynamic_t<EigMat>* = nullptr>
 inline Eigen::Matrix<value_type_t<EigMat>, Eigen::Dynamic, 1> rvalue(
-    EigMat&& x,
-    const cons_index_list<index_multi,
-                          cons_index_list<index_uni, nil_index_list>>& idxs,
-    const char* name = "ANON", int depth = 0) {
+    EigMat&& x, const char* name, const index_multi& row_idx,
+    index_uni col_idx) {
   math::check_range("matrix[multi, uni] column indexing", name, x.cols(),
-                    idxs.tail_.head_.n_);
+                    col_idx.n_);
   const auto& x_ref = stan::math::to_ref(x);
   Eigen::Matrix<value_type_t<EigMat>, Eigen::Dynamic, 1> x_ret(
-      idxs.head_.ns_.size());
-  for (int i = 0; i < idxs.head_.ns_.size(); ++i) {
+      row_idx.ns_.size());
+  for (int i = 0; i < row_idx.ns_.size(); ++i) {
     math::check_range("matrix[multi, uni] row indexing", name, x_ref.rows(),
-                      idxs.head_.ns_[i]);
-    x_ret.coeffRef(i)
-        = x_ref.coeff(idxs.head_.ns_[i] - 1, idxs.tail_.head_.n_ - 1);
+                      row_idx.ns_[i]);
+    x_ret.coeffRef(i) = x_ref.coeff(row_idx.ns_[i] - 1, col_idx.n_ - 1);
   }
   return x_ret;
 }
@@ -512,25 +484,23 @@ inline Eigen::Matrix<value_type_t<EigMat>, Eigen::Dynamic, 1> rvalue(
  *
  * @tparam EigMat An eigen matrix
  * @param[in] x Matrix to index.
- * @param[in] idxs Pair of multiple indexes.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] row_idx multi index for selecting rows.
+ * @param[in] col_idx multi index for selecting cols.
  * @return Result of indexing matrix.
  */
 template <typename EigMat, require_eigen_dense_dynamic_t<EigMat>* = nullptr>
-inline plain_type_t<EigMat> rvalue(
-    EigMat&& x,
-    const cons_index_list<index_multi,
-                          cons_index_list<index_multi, nil_index_list>>& idxs,
-    const char* name = "ANON", int depth = 0) {
+inline plain_type_t<EigMat> rvalue(EigMat&& x, const char* name,
+                                   const index_multi& row_idx,
+                                   const index_multi& col_idx) {
   const auto& x_ref = stan::math::to_ref(x);
-  const int rows = idxs.head_.ns_.size();
-  const int cols = idxs.tail_.head_.ns_.size();
+  const int rows = row_idx.ns_.size();
+  const int cols = col_idx.ns_.size();
   plain_type_t<EigMat> x_ret(rows, cols);
   for (int j = 0; j < cols; ++j) {
     for (int i = 0; i < rows; ++i) {
-      const int m = idxs.head_.ns_[i];
-      const int n = idxs.tail_.head_.ns_[j];
+      const int m = row_idx.ns_[i];
+      const int n = col_idx.ns_[j];
       math::check_range("matrix[multi,multi] row indexing", name, x_ref.rows(),
                         m);
       math::check_range("matrix[multi,multi] column indexing", name,
@@ -550,20 +520,16 @@ inline plain_type_t<EigMat> rvalue(
  * @tparam Mat An eigen matrix or `var_value<T>` whose inner type is an Eigen
  * matrix.
  * @param[in] x matrix.
- * @param[in] idxs Any index for the rows and a uni index for the columns
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] row_idx index for selecting rows.
+ * @param[in] col_idx uni index for selecting cols.
  */
 template <typename Mat, typename Idx, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(
-    Mat&& x,
-    const cons_index_list<Idx, cons_index_list<index_uni, nil_index_list>>&
-        idxs,
-    const char* name = "ANON", int depth = 0) {
+inline auto rvalue(Mat&& x, const char* name, const Idx& row_idx,
+                   index_uni col_idx) {
   math::check_range("matrix[..., uni] column indexing", name, x.cols(),
-                    idxs.tail_.head_.n_);
-  return rvalue(x.col(idxs.tail_.head_.n_ - 1), index_list(idxs.head_), name,
-                depth + 1);
+                    col_idx.n_);
+  return rvalue(x.col(col_idx.n_ - 1), name, row_idx);
 }
 
 /**
@@ -574,29 +540,26 @@ inline auto rvalue(
  *
  * @tparam EigMat An eigen matrix
  * @param[in] x Eigen matrix.
- * @param[in] idxs Any index for the rows and a multi index for the columns
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] row_idx index for selecting rows.
+ * @param[in] col_idx multi index for selecting cols.
  * @return Result of indexing matrix.
  */
 template <typename EigMat, typename Idx,
           require_eigen_dense_dynamic_t<EigMat>* = nullptr,
           require_not_same_t<std::decay_t<Idx>, index_uni>* = nullptr>
-inline plain_type_t<EigMat> rvalue(
-    EigMat&& x,
-    const cons_index_list<Idx, cons_index_list<index_multi, nil_index_list>>&
-        idxs,
-    const char* name = "ANON", int depth = 0) {
+inline plain_type_t<EigMat> rvalue(EigMat&& x, const char* name,
+                                   const Idx& row_idx,
+                                   const index_multi& col_idx) {
   const auto& x_ref = stan::math::to_ref(x);
-  const int rows = rvalue_index_size(idxs.head_, x_ref.rows());
-  const int cols = rvalue_index_size(idxs.tail_.head_, x_ref.cols());
-  plain_type_t<EigMat> x_ret(rows, idxs.tail_.head_.ns_.size());
-  for (int j = 0; j < idxs.tail_.head_.ns_.size(); ++j) {
-    const int n = idxs.tail_.head_.ns_[j];
+  const int rows = rvalue_index_size(row_idx, x_ref.rows());
+  const int cols = rvalue_index_size(col_idx, x_ref.cols());
+  plain_type_t<EigMat> x_ret(rows, col_idx.ns_.size());
+  for (int j = 0; j < col_idx.ns_.size(); ++j) {
+    const int n = col_idx.ns_[j];
     math::check_range("matrix[..., multi] column indexing", name, x_ref.cols(),
                       n);
-    x_ret.col(j)
-        = rvalue(x_ref.col(n - 1), index_list(idxs.head_), name, depth + 1);
+    x_ret.col(j) = rvalue(x_ref.col(n - 1), name, row_idx);
   }
   return x_ret;
 }
@@ -609,18 +572,14 @@ inline plain_type_t<EigMat> rvalue(
  * @tparam Mat An eigen matrix or `var_value<T>` whose inner type is an Eigen
  * matrix.
  * @param[in] x type
- * @param[in] idxs Any index for the rows and omni index for the columns
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
+ * @param[in] name Name of variable
+ * @param[in] row_idx index for selecting rows.
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename Mat, typename Idx, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(
-    Mat&& x,
-    const cons_index_list<Idx, cons_index_list<index_omni, nil_index_list>>&
-        idxs,
-    const char* name = "ANON", int depth = 0) {
-  return rvalue(std::forward<Mat>(x), index_list(idxs.head_), name, depth + 1);
+inline auto rvalue(Mat&& x, const char* name, const Idx& row_idx,
+                   index_omni /*col_idx*/) {
+  return rvalue(std::forward<Mat>(x), name, row_idx);
 }
 
 /**
@@ -633,22 +592,18 @@ inline auto rvalue(
  * matrix.
  * @tparam Idx An index.
  * @param[in] x type
- * @param[in] idxs Container holding a row index and an index from a minimum
- * index (inclusive) to the end of a container.
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
+ * @param[in] name Name of variable
+ * @param[in] row_idx index for selecting rows.
+ * @param[in] col_idx min index for selecting cols.
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename Mat, typename Idx, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(
-    Mat&& x,
-    const cons_index_list<Idx, cons_index_list<index_min, nil_index_list>>&
-        idxs,
-    const char* name = "ANON", int depth = 0) {
-  const auto col_size = x.cols() - (idxs.tail_.head_.min_ - 1);
+inline auto rvalue(Mat&& x, const char* name, const Idx& row_idx,
+                   index_min col_idx) {
+  const auto col_size = x.cols() - (col_idx.min_ - 1);
   math::check_range("matrix[..., min] column indexing", name, x.cols(),
-                    idxs.tail_.head_.min_);
-  return rvalue(x.rightCols(col_size), index_list(idxs.head_), name, depth + 1);
+                    col_idx.min_);
+  return rvalue(x.rightCols(col_size), name, row_idx);
 }
 
 /**
@@ -661,22 +616,17 @@ inline auto rvalue(
  * matrix.
  * @tparam Idx An index.
  * @param[in] x Eigen type
- * @param[in] idxs Index holding a row index and an index from the start of the
- * container up to the specified maximum index (inclusive).
- * @param[in] name Name of variable (default "ANON").
- * @param[in] depth Indexing depth (default 0).
+ * @param[in] name Name of variable
+ * @param[in] row_idx index for selecting rows.
+ * @param[in] col_idx max index for selecting cols.
  * @throw std::out_of_range If any of the indices are out of bounds.
  */
 template <typename Mat, typename Idx, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(
-    Mat&& x,
-    const cons_index_list<Idx, cons_index_list<index_max, nil_index_list>>&
-        idxs,
-    const char* name = "ANON", int depth = 0) {
+inline auto rvalue(Mat&& x, const char* name, const Idx& row_idx,
+                   index_max col_idx) {
   math::check_range("matrix[..., max] column indexing", name, x.cols(),
-                    idxs.tail_.head_.max_);
-  return rvalue(x.leftCols(idxs.tail_.head_.max_), index_list(idxs.head_), name,
-                depth + 1);
+                    col_idx.max_);
+  return rvalue(x.leftCols(col_idx.max_), name, row_idx);
 }
 
 /**
@@ -689,31 +639,28 @@ inline auto rvalue(
  * matrix.
  * @tparam Idx Type of index.
  * @param[in] x Matrix to index.
- * @param[in] idxs Any index for the rows and a min max index for the columns
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] row_idx index for selecting rows.
+ * @param[in] col_idx min max index for selecting cols.
  * @return Result of indexing matrix.
  */
 template <typename Mat, typename Idx, require_dense_dynamic_t<Mat>* = nullptr>
-inline auto rvalue(
-    Mat&& x,
-    const cons_index_list<Idx, cons_index_list<index_min_max, nil_index_list>>&
-        idxs,
-    const char* name = "ANON", int depth = 0) {
+inline auto rvalue(Mat&& x, const char* name, const Idx& row_idx,
+                   index_min_max col_idx) {
   math::check_range("matrix[..., min_max] min column indexing", name, x.cols(),
-                    idxs.tail_.head_.min_);
+                    col_idx.min_);
   math::check_range("matrix[..., min_max] max column indexing", name, x.cols(),
-                    idxs.tail_.head_.max_);
-  if (idxs.tail_.head_.is_ascending()) {
-    const auto col_start = idxs.tail_.head_.min_ - 1;
-    return rvalue(x.middleCols(col_start, idxs.tail_.head_.max_ - col_start),
-                  index_list(idxs.head_), name, depth + 1)
+                    col_idx.max_);
+  if (col_idx.is_ascending()) {
+    const auto col_start = col_idx.min_ - 1;
+    return rvalue(x.middleCols(col_start, col_idx.max_ - col_start), name,
+                  row_idx)
         .eval();
   } else {
-    const auto col_start = idxs.tail_.head_.max_ - 1;
-    return rvalue(internal::rowwise_reverse(x.middleCols(
-                      col_start, idxs.tail_.head_.min_ - col_start)),
-                  index_list(idxs.head_), name, depth + 1)
+    const auto col_start = col_idx.max_ - 1;
+    return rvalue(internal::rowwise_reverse(
+                      x.middleCols(col_start, col_idx.min_ - col_start)),
+                  name, row_idx)
         .eval();
   }
 }
@@ -729,20 +676,20 @@ inline auto rvalue(
  * @tparam T Type of list elements.
  * @tparam Idx Index list type for indexes after first index.
  * @param[in] v Container of list elements.
- * @param[in] idxs Index list beginning with single index.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] idx1 first index.
+ * @param[in] idxs remaining indices.
  * @return Result of indexing array.
  */
-template <typename StdVec, typename Idx,
+template <typename StdVec, typename... Idxs,
           require_std_vector_t<StdVec>* = nullptr>
-inline auto rvalue(StdVec&& v, const cons_index_list<index_uni, Idx>& idxs,
-                   const char* name = "ANON", int depth = 0) {
-  math::check_range("array[uni, ...] index", name, v.size(), idxs.head_.n_);
+inline auto rvalue(StdVec&& v, const char* name, index_uni idx1,
+                   const Idxs&... idxs) {
+  math::check_range("array[uni, ...] index", name, v.size(), idx1.n_);
   if (std::is_rvalue_reference<StdVec>::value) {
-    return rvalue(std::move(v[idxs.head_.n_ - 1]), idxs.tail_, name, depth + 1);
+    return rvalue(std::move(v[idx1.n_ - 1]), name, idxs...);
   } else {
-    return rvalue(v[idxs.head_.n_ - 1], idxs.tail_, name, depth + 1);
+    return rvalue(v[idx1.n_ - 1], name, idxs...);
   }
 }
 
@@ -753,18 +700,31 @@ inline auto rvalue(StdVec&& v, const cons_index_list<index_uni, Idx>& idxs,
  * Types:  std::vector<T>[uni] : T
  *
  * @tparam StdVec a standard vector
- * @param[in] c Container of list elements.
- * @param[in] idxs Index list beginning with single index.
+ * @param[in] v Container of list elements.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] idx single index
  * @return Result of indexing array.
  */
 template <typename StdVec, require_std_vector_t<StdVec>* = nullptr>
-inline auto rvalue(StdVec&& v,
-                   const cons_index_list<index_uni, nil_index_list>& idxs,
-                   const char* name = "ANON", int depth = 0) {
-  math::check_range("array[uni, ...] index", name, v.size(), idxs.head_.n_);
-  return v[idxs.head_.n_ - 1];
+inline auto rvalue(StdVec&& v, const char* name, index_uni idx) {
+  math::check_range("array[uni, ...] index", name, v.size(), idx.n_);
+  if (std::is_rvalue_reference<StdVec>::value) {
+    return std::move(v[idx.n_ - 1]);
+  } else {
+    return v[idx.n_ - 1];
+  }
+}
+
+template <typename StdVec, require_std_vector_t<StdVec>* = nullptr>
+inline auto& rvalue(StdVec& v, const char* name, index_uni idx) {
+  math::check_range("array[uni, ...] index", name, v.size(), idx.n_);
+  return v[idx.n_ - 1];
+}
+
+template <typename StdVec, require_std_vector_t<StdVec>* = nullptr>
+inline const auto& rvalue(const StdVec& v, const char* name, index_uni idx) {
+  math::check_range("array[uni, ...] index", name, v.size(), idx.n_);
+  return v[idx.n_ - 1];
 }
 
 /**
@@ -779,30 +739,30 @@ inline auto rvalue(StdVec&& v,
  * @tparam Idx1 Index list type for first index.
  * @tparam Idx2 Index list type for second index index.
  * @param[in] v Container of list elements.
- * @param[in] idxs Index list beginning with multiple index.
  * @param[in] name String form of expression being evaluated.
- * @param[in] depth Depth of indexing dimension.
+ * @param[in] idx1 first index
+ * @param[in] idxs remaining indices
  * @return Result of indexing array.
  */
-template <typename StdVec, typename Idx1, typename Idx2,
-          require_std_vector_t<StdVec>* = nullptr>
-inline auto rvalue(StdVec&& v, const cons_index_list<Idx1, Idx2>& idxs,
-                   const char* name = "ANON", int depth = 0) {
+template <typename StdVec, typename Idx1, typename... Idxs,
+          require_std_vector_t<StdVec>* = nullptr,
+          require_not_same_t<Idx1, index_uni>* = nullptr>
+inline auto rvalue(StdVec&& v, const char* name, Idx1 idx1,
+                   const Idxs&... idxs) {
   using inner_type = plain_type_t<decltype(
-      rvalue(v[rvalue_at(0, idxs.head_) - 1], idxs.tail_))>;
+      rvalue(v[rvalue_at(0, idx1) - 1], name, idxs...))>;
   std::vector<inner_type> result;
-  const int index_size = rvalue_index_size(idxs.head_, v.size());
+  const int index_size = rvalue_index_size(idx1, v.size());
   if (index_size > 0) {
     result.reserve(index_size);
   }
   for (int i = 0; i < index_size; ++i) {
-    const int n = rvalue_at(i, idxs.head_);
+    const int n = rvalue_at(i, idx1);
     math::check_range("array[..., ...] index", name, v.size(), n);
     if (std::is_rvalue_reference<StdVec>::value) {
-      result.emplace_back(
-          rvalue(std::move(v[n - 1]), idxs.tail_, name, depth + 1));
+      result.emplace_back(rvalue(std::move(v[n - 1]), name, idxs...));
     } else {
-      result.emplace_back(rvalue(v[n - 1], idxs.tail_, name, depth + 1));
+      result.emplace_back(rvalue(v[n - 1], name, idxs...));
     }
   }
   return result;
