@@ -4,18 +4,21 @@ cp -r . $PREFIX/bin/cmdstan
 cd $PREFIX/bin/cmdstan
 
 echo "TBB_CXX_TYPE=${c_compiler}"  >> make/local
+echo "TBB_INTERFACE_NEW=true" >> make/local
+echo "TBB_INC=${PREFIX}/include/" >> make/local
+echo "TBB_LIB=${PREFIX}/lib/" >> make/local
+echo "PRECOMPILED_HEADERS=false" >> make/local
+
+cat make/local
 
 make clean-all
 
 make build -j${CPU_COUNT}
 
-# delete precompiled header from build
-rm stan/src/stan/model/model_header.hpp.gch
-
 # set up an alias. see https://github.com/stan-dev/cmdstan/issues/1024
 echo "#!/bin/bash" > "${PREFIX}/bin/cmdstan_model"
 echo "fname=\${1%.stan}" >> "${PREFIX}/bin/cmdstan_model"
-echo "make -C ${PREFIX}/bin/cmdstan \$(realpath --relative-to=${PREFIX}/bin/cmdstan \$fname) \"\${@:2}\"" >> "${PREFIX}/bin/cmdstan_model"
+echo "make -C ${PREFIX}/bin/cmdstan \$(realpath --relative-to=${PREFIX}/bin/cmdstan \$fname)" >> "${PREFIX}/bin/cmdstan_model"
 chmod +x "${PREFIX}/bin/cmdstan_model"
 
 # activate script
